@@ -14,6 +14,7 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
+import importlib
 import logging
 import platform
 import socket
@@ -41,8 +42,15 @@ from cdpcli.compat import six
 from cdpcli.config import Config
 from cdpcli.endpoint import EndpointCreator
 from cdpcli.endpoint import EndpointResolver
+from cdpcli.exceptions import ExtensionImportError
+from cdpcli.exceptions import InvalidConfiguredFormFactor
+from cdpcli.exceptions import ProfileNotFound
+from cdpcli.exceptions import WrongOpFormFactorError
+from cdpcli.exceptions import WrongSvcFormFactorError
 from cdpcli.extensions.arguments import OverrideRequiredArgsArgument
 from cdpcli.extensions.cliinputjson import add_cli_input_json
+from cdpcli.extensions.configure.classify import ClassifyDeployment
+from cdpcli.extensions.configure.classify import DeploymentType
 from cdpcli.extensions.configure.configure import ConfigureCommand
 from cdpcli.extensions.generatecliskeleton import add_generate_skeleton
 from cdpcli.extensions.interactivelogin import LoginCommand
@@ -84,8 +92,9 @@ class CLIDriver(object):
         self._available_services = self._loader.list_available_services()
         self._command_table = self._build_command_table()
         self._argument_table = self._build_argument_table()
+        self._context = Context()
         self._client_creator = ClientCreator(self._loader,
-                                             Context(),
+                                             self._context,
                                              self._endpoint_creator,
                                              self._user_agent_header,
                                              self._response_parser_factory,
@@ -101,6 +110,7 @@ class CLIDriver(object):
         parsed_args, remaining = parser.parse_known_args(args)
         try:
             self._handle_top_level_args(parsed_args)
+            self._filter_command_table_for_form_factor(parsed_args)
             self._warn_for_old_python()
             self._warn_for_non_public_release()
             return command_table[parsed_args.command](
@@ -135,8 +145,71 @@ class CLIDriver(object):
         LoginCommand.add_command(commands)
         LogoutCommand.add_command(commands)
         RefdocCommand.add_command(commands)
+        commands = OrderedDict(sorted(commands.items()))
         return commands
 
+    def _filter_command_table_for_form_factor(self, parsed_args):
+        """
+        Replaces services and operations in the command table that do not apply
+        to the current form factor with stubs that error out when called.
+        """
+
+        # Find the form factor based on:
+        # 1. the form factor explicitly specified by --form-factor, or else
+        # 2. the configured form factor, or else
+        # 3. the explicit endpoint URL, or else
+        # 4. the configured CDP endpoint URL.
+        if parsed_args.form_factor:
+            form_factor = parsed_args.form_factor
+        else:
+            try:
+                form_factor = self._context.get_scoped_config().get('form_factor', None)
+            except ProfileNotFound:
+                form_factor = None
+            valid_form_factors = [dt.value for dt in list(DeploymentType)]
+            if form_factor and form_factor not in valid_form_factors:
+                raise InvalidConfiguredFormFactor(
+                    form_factor=form_factor,
+                    valid_form_factors=valid_form_factors)
+            if not form_factor:
+                endpoint_url = parsed_args.endpoint_url
+                if not endpoint_url:
+                    try:
+                        endpoint_url = self._context.get_scoped_config(). \
+                            get(EndpointResolver.CDP_ENDPOINT_URL_KEY_NAME, None)
+                    except ProfileNotFound:
+                        endpoint_url = None
+                form_factor = \
+                    ClassifyDeployment(endpoint_url).get_deployment_type().value
+        LOG.debug("Current form factor is {}".format(form_factor))
+
+        for command in list(self._command_table.keys()):
+            try:
+                # If the service does not apply to the current form factor,
+                # filter it out.
+                service_model = self._command_table[command].service_model
+                service_form_factors = service_model.form_factors
+                if form_factor not in service_form_factors:
+                    self._command_table[command] = \
+                        FilteredServiceCommand(self, command, form_factor,
+                                               service_form_factors)
+                else:
+                    for operation_name in service_model.operation_names:
+                        # If the operation does not apply to the current form
+                        # factor, filter it out.
+                        operation_model = service_model.operation_model(operation_name)
+                        operation_form_factors = operation_model.form_factors
+                        if not operation_form_factors:
+                            operation_form_factors = service_form_factors
+                        if form_factor not in operation_form_factors:
+                            self._command_table[command]. \
+                                filter_operation(operation_name, form_factor,
+                                                 operation_form_factors)
+
+            except AttributeError:
+                # not a service model, so available in all form factors
+                pass
+
     def _get_argument_table(self):
         return self._argument_table
 
@@ -155,6 +228,7 @@ class CLIDriver(object):
 
     def get_service_model(self, service_name):
         service_data = self._loader.load_service_data(service_name)
+        service_data['paths'] = OrderedDict(sorted(service_data.get('paths', {}).items()))
         return ServiceModel(service_data, service_name=service_name)
 
     def _create_help_command(self):
@@ -178,23 +252,23 @@ class CLIDriver(object):
         command_table['help'] = self._create_help_command()
         cli_data = self._get_cli_data()
         parser = MainArgParser(
-          command_table,
-          VERSION,
-          cli_data.get('description', None),
-          self._get_argument_table())
+            command_table,
+            VERSION,
+            cli_data.get('description', None),
+            self._get_argument_table())
         return parser
 
     def _create_cli_argument(self, option_name, option_params):
         return CustomArgument(
-          option_name,
-          help_text=option_params.get('help', ''),
-          dest=option_params.get('dest'),
-          default=option_params.get('default'),
-          action=option_params.get('action'),
-          required=option_params.get('required'),
-          choices=option_params.get('choices'),
-          cli_type_name=option_params.get('type'),
-          hidden=option_params.get('hidden', False))
+            option_name,
+            help_text=option_params.get('help', ''),
+            dest=option_params.get('dest'),
+            default=option_params.get('default'),
+            action=option_params.get('action'),
+            required=option_params.get('required'),
+            choices=option_params.get('choices'),
+            cli_type_name=option_params.get('type'),
+            hidden=option_params.get('hidden', False))
 
     def _handle_top_level_args(self, args):
         if args.profile:
@@ -252,13 +326,20 @@ class CLIDriver(object):
 
     def _warn_for_non_public_release(self):
         if RELEASE != 'PUBLIC':
-            LOG.warn('You are running a {0} release of the CDP CLI, which '
+            if RELEASE == 'INTERNAL':
+                article = 'an'
+            else:
+                article = 'a'
+            LOG.warn('You are running {0} {1} release of the CDP CLI, which '
                      'has different capabilities from the standard public '
                      'release. Find the public release at: '
-                     'https://pypi.org/project/cdpcli/'.format(RELEASE))
+                     'https://pypi.org/project/cdpcli/'.format(article, RELEASE))
 
 
 class ServiceCommand(CLICommand):
+    """
+    A top-level CLI command, corresponding to an API service.
+    """
 
     def __init__(self, clidriver, name):
         self._clidriver = clidriver
@@ -322,15 +403,31 @@ class ServiceCommand(CLICommand):
             __import__('cdpcli.extensions.%s.register' % self._name,
                        fromlist=['register']).register(command_table)
         except ImportError as err:
-            py3_err = "No module named 'cdpcli.extensions.%s'" % self._name
+            py3_err_1 = "No module named 'cdpcli.extensions.%s'" % self._name
+            py3_err_2 = "No module named 'cdpcli.extensions.%s.register'" % self._name
             py2_err = "No module named %s.register" % self._name
-            if py2_err not in str(err) and py3_err not in str(err):
+            if py2_err not in str(err) and \
+               py3_err_1 not in str(err) and \
+               py3_err_2 not in str(err):
                 # Looks like a different error than missing extensions.
                 LOG.warn("Failed to import service (%s) extension: %s", self._name, err)
             pass
         self._add_lineage(command_table)
         return command_table
 
+    def filter_operation(self, operation_name, form_factor, operation_form_factors):
+        """
+        Replace the named operation in this command's command table with a
+        filtered one.
+        """
+        command_table = self._get_command_table()
+        cli_name = xform_name(operation_name, '-')
+        command_table[cli_name] = FilteredServiceOperation(
+            name=cli_name,
+            parent_name=self._name,
+            form_factor=form_factor,
+            operation_form_factors=operation_form_factors)
+
     def _add_lineage(self, command_table):
         for command in command_table:
             command_obj = command_table[command]
@@ -354,6 +451,26 @@ class ServiceCommand(CLICommand):
             operations_table=command_table, service_name=self._name)
 
 
+class FilteredServiceCommand(ServiceCommand):
+    """
+    A stub service command that fails when run due to being under the wrong
+    CLI form factor.
+    """
+
+    def __init__(self, clidriver, name, form_factor, service_form_factors):
+        super().__init__(clidriver, name)
+        self._clidriver = clidriver
+        self._name = name
+        self._form_factor = form_factor
+        self._service_form_factors = service_form_factors
+
+    def __call__(self, client_creator, args, parsed_globals):
+        raise WrongSvcFormFactorError(
+            service_name=self._name,
+            form_factor=self._form_factor,
+            service_form_factors=', '.join(self._service_form_factors))
+
+
 class ServiceOperation(object):
     ARG_TYPES = {
         LIST_TYPE: ListArgument,
@@ -401,6 +518,9 @@ class ServiceOperation(object):
         return self._arg_table
 
     def __call__(self, client_creator, args, parsed_globals):
+        # Handle extensions first, so OverrideRequiredArgs (CliInputJson,
+        # GenerateCliSkeleton, etc) could have a chance to run.
+        self._handle_extensions()
         # We need to handle overriding required arguments before we create
         # the parser as the parser will parse the arguments and decide which
         # argument is required before we have a chance to modify the argument
@@ -425,30 +545,7 @@ class ServiceOperation(object):
                                        parsed_globals)
         call_parameters = self._build_call_parameters(parsed_args,
                                                       self.arg_table)
-
-        # The TLS verification value can be a boolean or a CA_BUNDLE path. This
-        # is a little odd, but ultimately comes from the python HTTP requests
-        # library we're using.
-        tls_verification = parsed_globals.verify_tls
-        ca_bundle = getattr(parsed_globals, 'ca_bundle', None)
-        if parsed_globals.verify_tls and ca_bundle is not None:
-            tls_verification = ca_bundle
-
-        # Retrieve values passed for extra client configuration.
-        config_kwargs = {}
-        if parsed_globals.read_timeout is not None:
-            config_kwargs['read_timeout'] = int(parsed_globals.read_timeout)
-        if parsed_globals.connect_timeout is not None:
-            config_kwargs['connect_timeout'] = int(parsed_globals.connect_timeout)
-        config = Config(**config_kwargs)
-
-        client = client_creator.create_client(
-            self._operation_model.service_model.service_name,
-            parsed_globals.endpoint_url,
-            tls_verification,
-            client_creator.context.get_credentials(parsed_globals),
-            client_config=config)
-        return self._invoke_operation_callers(client,
+        return self._invoke_operation_callers(client_creator,
                                               call_parameters,
                                               parsed_args,
                                               parsed_globals)
@@ -522,15 +619,56 @@ class ServiceOperation(object):
                 cli_argument.override_required_args(argument_table, args)
                 self._operation_callers.insert(0, cli_argument)
 
+    def _handle_extensions(self):
+        if self._operation_model.extensions:
+            # Iterate in reversed order to keep the execution order:
+            # First extension should run first.
+            for ext_name in reversed(self._operation_model.extensions):
+                try:
+                    module = importlib.import_module('cdpcli.extensions.%s' % ext_name)
+                    register_func = getattr(module, 'register')
+                    register_func(self._operation_callers,
+                                  self._operation_model)
+                except Exception as err:
+                    raise ExtensionImportError(ext_name=ext_name, err=err)
+
     def _invoke_operation_callers(self,
-                                  client,
+                                  client_creator,
                                   call_parameters,
                                   parsed_args,
                                   parsed_globals):
+        def _create_client(service_name):
+            # The TLS verification value can be a boolean or a CA_BUNDLE path. This
+            # is a little odd, but ultimately comes from the python HTTP requests
+            # library we're using.
+            tls_verification = parsed_globals.verify_tls
+            ca_bundle = getattr(parsed_globals, 'ca_bundle', None)
+            if parsed_globals.verify_tls and ca_bundle is not None:
+                tls_verification = ca_bundle
+
+            # Retrieve values passed for extra client configuration.
+            config_kwargs = {}
+            if parsed_globals.read_timeout is not None:
+                config_kwargs['read_timeout'] = int(parsed_globals.read_timeout)
+            if parsed_globals.connect_timeout is not None:
+                config_kwargs['connect_timeout'] = int(parsed_globals.connect_timeout)
+            config = Config(**config_kwargs)
+
+            client = client_creator.create_client(
+                service_name,
+                parsed_globals.endpoint_url,
+                parsed_globals.cdp_region,
+                tls_verification,
+                client_creator.context.get_credentials(parsed_globals),
+                client_config=config)
+            return client
+
         for operation_caller in self._operation_callers:
+            # Create a new client for each operation_caller because parsed_args and
+            # parsed_globals could be changed in each iteration.
             if operation_caller.invoke(
-                    client,
-                    self._operation_model.name,
+                    _create_client,
+                    self._operation_model,
                     call_parameters,
                     parsed_args,
                     parsed_globals) is False:
@@ -538,14 +676,36 @@ class ServiceOperation(object):
         return 0
 
 
+class FilteredServiceOperation(ServiceOperation):
+    """
+    A stub service operation that fails when run due to being under the wrong
+    CLI form factor.
+    """
+
+    def __init__(self, name, parent_name, form_factor, operation_form_factors):
+        super().__init__(name, parent_name, operation_caller=None, operation_model=None)
+        self._form_factor = form_factor
+        self._operation_form_factors = operation_form_factors
+
+    def __call__(self, client_creator, args, parsed_globals):
+        raise WrongOpFormFactorError(
+            operation_name=self._name,
+            service_name=self._parent_name,
+            form_factor=self._form_factor,
+            operation_form_factors=', '.join(self._operation_form_factors))
+
+
 class CLIOperationCaller(object):
 
     def invoke(self,
-               client,
-               operation_name,
+               client_creator,
+               operation_model,
                parameters,
                parsed_args,
                parsed_globals):
+        service_name = operation_model.service_model.service_name
+        operation_name = operation_model.name
+        client = client_creator(service_name)
         py_operation_name = xform_name(operation_name)
         if client.can_paginate(py_operation_name) and parsed_globals.paginate:
             response = client.get_paginator(
